package cn.com.infosec.crypto.macs;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Mac;
import cn.com.infosec.crypto.modes.CBCBlockCipher;
import cn.com.infosec.crypto.paddings.BlockCipherPadding;

public class CBCBlockCipherMac implements Mac {
   private byte[] mac;
   private byte[] buf;
   private int bufOff;
   private BlockCipher cipher;
   private BlockCipherPadding padding;
   private int macSize;

   public CBCBlockCipherMac(BlockCipher cipher) {
      this(cipher, cipher.getBlockSize() * 8 / 2, (BlockCipherPadding)null);
   }

   public CBCBlockCipherMac(BlockCipher cipher, BlockCipherPadding padding) {
      this(cipher, cipher.getBlockSize() * 8 / 2, padding);
   }

   public CBCBlockCipherMac(BlockCipher cipher, int macSizeInBits) {
      this(cipher, macSizeInBits, (BlockCipherPadding)null);
   }

   public CBCBlockCipherMac(BlockCipher cipher, int macSizeInBits, BlockCipherPadding padding) {
      if (macSizeInBits % 8 == 0) {
         this.cipher = new CBCBlockCipher(cipher);
         this.padding = padding;
         this.macSize = macSizeInBits / 8;
         this.mac = new byte[cipher.getBlockSize()];
         this.buf = new byte[cipher.getBlockSize()];
         this.bufOff = 0;
      } else {
         throw new IllegalArgumentException("MAC size must be multiple of 8");
      }
   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName();
   }

   public void init(CipherParameters params) {
      this.reset();
      this.cipher.init(true, params);
   }

   public int getMacSize() {
      return this.macSize;
   }

   public void update(byte in) {
      byte[] var2;
      if (this.bufOff == (var2 = this.buf).length) {
         this.cipher.processBlock(var2, 0, this.mac, 0);
         this.bufOff = 0;
      }

      byte[] var10000 = this.buf;
      int var3;
      this.bufOff = (var3 = this.bufOff) + 1;
      var10000[var3] = in;
   }

   public void update(byte[] in, int inOff, int len) {
      if (len < 0) {
         throw new IllegalArgumentException("Can't have a negative input length!");
      } else {
         int var4;
         int var5;
         int var6;
         int var10000;
         if (len > (var6 = (var4 = this.cipher.getBlockSize()) - (var5 = this.bufOff))) {
            var10000 = inOff;
            int var10002 = len;
            System.arraycopy(in, inOff, this.buf, var5, var6);
            byte[] inOff1 = this.buf;
            byte[] len1 = this.mac;
            this.cipher.processBlock(inOff1, 0, len1, 0);
            this.bufOff = 0;
            inOff = var10002 - var6;

            for(len = var10000 + var6; inOff > var4; len += var4) {
               this.cipher.processBlock(in, len, this.mac, 0);
               inOff -= var4;
            }
         } else {
            var10000 = inOff;
            inOff = len;
            len = var10000;
         }

         CBCBlockCipherMac var10 = this;
         CBCBlockCipherMac var10001 = this;
         CBCBlockCipherMac var10004 = this;
         byte[] var7 = this.buf;
         System.arraycopy(in, len, var7, var10004.bufOff, inOff);
         var10.bufOff = var10001.bufOff + inOff;
      }
   }

   public int doFinal(byte[] out, int outOff) {
      int var3 = this.cipher.getBlockSize();
      byte[] var4;
      byte[] var6;
      int var7;
      if (this.padding == null) {
         while((var7 = this.bufOff) < var3) {
            this.buf[var7] = 0;
            this.bufOff = var7 + 1;
         }
      } else {
         if (this.bufOff == var3) {
            var6 = this.buf;
            var4 = this.mac;
            this.cipher.processBlock(var6, 0, var4, 0);
            this.bufOff = 0;
         }

         var6 = this.buf;
         this.padding.addPadding(var6, this.bufOff);
      }

      CBCBlockCipherMac var10000 = this;
      CBCBlockCipherMac var10001 = this;
      var6 = this.buf;
      var4 = this.mac;
      this.cipher.processBlock(var6, 0, var4, 0);
      byte[] var10002 = this.mac;
      int var5 = this.macSize;
      System.arraycopy(var10002, 0, out, outOff, var5);
      var10001.reset();
      return var10000.macSize;
   }

   public void reset() {
      byte[] var2;
      for(int var1 = 0; var1 < (var2 = this.buf).length; ++var1) {
         var2[var1] = 0;
      }

      this.bufOff = 0;
      this.cipher.reset();
   }
}
